
// LISTING 3:

// OUTPORT.CPP - Implementation file for class OUTPORT

// Eric Horner, Sundstrand Research Dept., Rockford, IL. May 1990

#include "outport.hpp"
#include <stdio.h>

// constructor
outport::outport(char *address)
{
    maxbit = 7;
    port_address = address;
}

// set one bit 
boolean outport::set_bit(int bit_number)
{
    if ((bit_number < 0) || (bit_number > maxbit))
        return FALSE;                       // out of range error
    char zerobit = 0x01;
    *port_address |= (zerobit << bit_number);
    return TRUE;
}

// set several bits
boolean outport::set_bits(int A, int B, int C, int D)
{
    if (!set_bit(A) ) return FALSE;
    if (!set_bit(B) ) return FALSE;
    if (!set_bit(C) ) return FALSE;
    if (!set_bit(D) ) return FALSE;
    return TRUE;
}

// set a range of bits
boolean outport::set_bits(int start, int end)
{
    for (int i = start; i <= end; i++)
        if (!set_bit(i) ) return FALSE;
    return TRUE;
}

// set all bits
boolean outport::set_all()
{
    for (int i = 0; i <= maxbit; i++)    // can not be out of range!
        if (!set_bit(i) ) return FALSE;
    return TRUE;
}


// clear one bit 
boolean outport::clear_bit(int bit_number)
{
    if ((bit_number < 0) || (bit_number > maxbit))
        return FALSE;                       // out of range error
    char zerobit = ~(0x01 << bit_number);
    *port_address &= zerobit;
    return TRUE;
}

// clear several bits
boolean outport::clear_bits(int A, int B, int C, int D)
{
    if (!clear_bit(A) ) return FALSE;
    if (!clear_bit(B) ) return FALSE;
    if (!clear_bit(C) ) return FALSE;
    if (!clear_bit(D) ) return FALSE;
    return TRUE;
}

// clear a range of bits
boolean outport::clear_bits(int start, int end)
{
    for (int i = start; i <= end; i++)
        if (!clear_bit(i) ) return FALSE;
    return TRUE;
}

// clear all bits
boolean outport::clear_all()
{
    for (int i = 0; i <= maxbit; i++)    // can not be out of range!
        if (!clear_bit(i) ) return FALSE;
    return TRUE;
}


// toggle one bit 
boolean outport::toggle_bit(int bit_number)
{
    if ((bit_number < 0) || (bit_number > maxbit))
        return FALSE;                       // out of range error
    char zerobit = (0x01 << bit_number);
    *port_address ^= zerobit;
    return TRUE;
}

// toggle several bits
boolean outport::toggle_bits(int A, int B, int C, int D)
{
    if (!toggle_bit(A) ) return FALSE;
    if (!toggle_bit(B) ) return FALSE;
    if (!toggle_bit(C) ) return FALSE;
    if (!toggle_bit(D) ) return FALSE;
    return TRUE;
}

// toggle a range of bits
boolean outport::toggle_bits(int start, int end)
{
    for (int i = start; i <= end; i++)
        if (!toggle_bit(i) ) return FALSE;
    return TRUE;
}

// toggle all bits
boolean outport::toggle_all()
{
    for (int i = 0; i <= maxbit; i++)    // can not be out of range!
        if (!toggle_bit(i) ) return FALSE;
    return TRUE;
}

// dump bit values of port to the screen, for debug
void outport::print_port()
{
    printf("\nBIT Number: 7 6 5 4 3 2 1 0");
    printf("\nSTATE :     ");
    if (*port_address & 0x80) printf("1 "); else printf("0 ");
    if (*port_address & 0x40) printf("1 "); else printf("0 ");
    if (*port_address & 0x20) printf("1 "); else printf("0 ");
    if (*port_address & 0x10) printf("1 "); else printf("0 ");
    if (*port_address & 0x08) printf("1 "); else printf("0 ");
    if (*port_address & 0x04) printf("1 "); else printf("0 ");
    if (*port_address & 0x02) printf("1 "); else printf("0 ");
    if (*port_address & 0x01) printf("1 "); else printf("0 ");
    printf("\n\n");
}

